%% MASTERS DEGREE - IMU test for SINTEF
 % Author:  Aslak J. Strand
 % Purpose: Testing multiple MPU6050 IMU chips in paralell
%% Clearing workspace:
clear all; 
close all; 
clc

pause on

%% GLOBALS
sample = 0;


%% INPUT
fprintf('~Select Test# for new filename (test<x>)~')
testNr = input('\nInput test#: ');


fprintf('\n~Getting Stuff Ready~\n')
runTime = input('Run time [s]: ');


fprintf('\n~Which HUB are you using?~')
fprintf('\n[1] Single USB-C to USB-3 converter')
fprintf('\n[2] HyperDrive 2x USB-3')
fprintf('\n[3] Fake Apple 4x USB-2')
fprintf('\n[4] SINTEF 4x DELTACO USB-3')
HUB     = input('\nSelect your HUB: ');

switch HUB
    case 1
        portNames = ["/dev/cu.usbmodem14501"];
    case 2
        portNames = ["/dev/cu.usbmodem1411301", "/dev/cu.usbmodem1411401"];
    case 3
        portNames = ["/dev/cu.usbmodem145101", "/dev/cu.usbmodem145201", 
            "/dev/cu.usbmodem145301", "/dev/cu.usbmodem145401"];
    case 4
        portNames = ["/dev/cu.usbmodem145101", "/dev/cu.usbmodem145201", 
            "/dev/cu.usbmodem145301", "/dev/cu.usbmodem145401"];
end



clc;


%% USB connection to ARDUINO(s)
fprintf('~Trying to connect to the Arduinos~\n')

a = 0;
try
    ard1 = arduino(portNames(1), 'Uno');
    fprintf('Connected to ARDUINO-1 @ %s\n', ard1.Port)
    a = a+1;
catch errno
    fprintf('Failed to connect to ARDUINO-1 with error:\n%s\n',errno.identifier)
    fprintf('Try to run <<arduinosetup>> from console.\n\n')
end

try
    ard2 = arduino(portNames(2), 'Uno');
    fprintf('Connected to ARDUINO-2 @ %s\n', ard2.Port)
    a = a+1;
catch errno
    fprintf('Failed to connect to ARDUINO-2 with error:\n%s\n',errno.identifier)
    fprintf('Try to run <<arduinosetup>> from console.\n\n')
end

try
    ard3 = arduino(portNames(3), 'Uno');
    fprintf('Connected to ARDUINO-3 @ %s\n', ard3.Port)
    a = a+1;
catch errno
    fprintf('Failed to connect to ARDUINO-3 with error:\n%s\n',errno.identifier)
    fprintf('Try to run <<arduinosetup>> from console.\n\n')
end

try
    ard4 = arduino(portNames(4), 'Uno');
    fprintf('Connected to ARDUINO-4 @ %s\n', ard4.Port)
    a = a+1;
catch errno
    fprintf('Failed to connect to ARDUINO-4 with error:\n%s\n',errno.identifier)
    fprintf('Try to run <<arduinosetup>> from console.\n\n')
end

try
    ard5 = arduino(portNames(5), 'Uno');
    fprintf('Connected to ARDUINO-5 @ %s\n\n', ard5.Port)
    a = a+1;
catch errno
    fprintf('Failed to connect to ARDUINO-5 with error:\n%s\n',errno.identifier)
    fprintf('Try to run <<arduinosetup>> from console.\n\n')
end



%% I2C connection to IMU(s)
fprintf('~Trying to connect to the IMUs~\n')
i = 0;
if a >= 1
    try
        imu1 = mpu6050(ard1,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
        fprintf('Connected to IMU-1 address: 0x%s\n', dec2hex(imu1.I2CAddress))
        i = i+1;
    catch errno
        fprintf('Failed to connect to IMU-1 with error:\n%s\n',errno.identifier)
    end
end

if a >= 2
    try
        imu2 = mpu6050(ard2,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
        fprintf('Connected to IMU-2 address: 0x%s\n', dec2hex(imu2.I2CAddress))
        i = i+1;
    catch errno
        fprintf('Failed to connect to IMU-2 with error:\n%s\n',errno.identifier)
    end
end

if a >= 3
    try
        imu3 = mpu6050(ard3,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
        fprintf('Connected to IMU-3 address: 0x%s\n', dec2hex(imu3.I2CAddress))
        i = i+1;
    catch errno
        fprintf('Failed to connect to IMU-3 with error:\n%s\n',errno.identifier)
    end
end

if a >= 4
    try
        imu4 = mpu6050(ard4,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
        fprintf('Connected to IMU-4 address: 0x%s\n', dec2hex(imu4.I2CAddress))
        i = i+1;
    catch errno
        fprintf('Failed to connect to IMU-4 with error:\n%s\n',errno.identifier)
    end
end

if a >= 5
    try
        imu5 = mpu6050(ard5,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
        fprintf('Connected to IMU-5 address: 0x%s\n', dec2hex(imu5.I2CAddress))
        i = i+1;
    catch errno
        fprintf('Failed to connect to IMU-5 with error:\n%s\n',errno.identifier)
    end
end


%% FIGURES 
%ardArr = zeros(1,numIMUs);
%imuArr = zeros(1,numIMUs);

if i >= 1
    figure('Name', 'IMU #1','NumberTitle','off');
    subplot(1,2,1)
        title('Accelerometer');
        axl1 = animatedline('Color','r');
        ayl1 = animatedline('Color','b');
        azl1 = animatedline('Color','g');
        legend('Ax','Ay','Az')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('m/s2');

    subplot(1,2,2)
        title('Gyro');
        gxl1 = animatedline('Color','r');
        gyl1 = animatedline('Color','b');
        gzl1 = animatedline('Color','g');
        legend('Gx','Gy','Gz')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('°/s');
        
    movegui(1,[200,600])
end

if i >= 2
    figure('Name', 'IMU #2','NumberTitle','off');
    subplot(1,2,1)
        title('Accelerometer');
        axl2 = animatedline('Color','r');
        ayl2 = animatedline('Color','b');
        azl2 = animatedline('Color','g');
        legend('Ax','Ay','Az')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('m/s2');

    subplot(1,2,2)
        title('Gyro');
        gxl2 = animatedline('Color','r');
        gyl2 = animatedline('Color','b');
        gzl2 = animatedline('Color','g');
        legend('Gx','Gy','Gz')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('°/s');
    
    movegui(2,[750,600])
end

if i >= 3
    figure('Name', 'IMU #3','NumberTitle','off');
    subplot(1,2,1)
        title('Accelerometer');
        axl3 = animatedline('Color','r');
        ayl3 = animatedline('Color','b');
        azl3 = animatedline('Color','g');
        legend('Ax','Ay','Az')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('m/s2');

    subplot(1,2,2)
        title('Gyro');
        gxl3 = animatedline('Color','r');
        gyl3 = animatedline('Color','b');
        gzl3 = animatedline('Color','g');
        legend('Gx','Gy','Gz')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('°/s');
        
    movegui(3,[200,100])
end

if i >= 4
    figure('Name', 'IMU #4','NumberTitle','off');
    subplot(1,2,1)
        title('Accelerometer');
        axl4 = animatedline('Color','r');
        ayl4 = animatedline('Color','b');
        azl4 = animatedline('Color','g');
        legend('Ax','Ay','Az')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('m/s2');

    subplot(1,2,2)
        title('Gyro');
        gxl4 = animatedline('Color','r');
        gyl4 = animatedline('Color','b');
        gzl4 = animatedline('Color','g');
        legend('Gx','Gy','Gz')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('°/s');
        
    movegui(4,[750,100])
end

if i >= 5
    figure('Name', 'IMU #5','NumberTitle','off');
    subplot(1,2,1)
        title('Accelerometer');
        axl5 = animatedline('Color','r');
        ayl5 = animatedline('Color','b');
        azl5 = animatedline('Color','g');
        legend('Ax','Ay','Az')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('m/s2');

    subplot(1,2,2)
        title('Gyro');
        gxl5 = animatedline('Color','r');
        gyl5 = animatedline('Color','b');
        gzl5 = animatedline('Color','g');
        legend('Gx','Gy','Gz')
        %xlim([0 runTime/sample])
        xlabel('Time [s]');
        ylabel('°/s');
        
    movegui(5,[1300,600])
end

%% INITIATE clock & Arrays
numIMUs = i;

time = clock;
strt = time;
stop = time;

if (stop(6) + runTime) > 60
    stop(5) = stop(5) + round( (stop(6) + runTime) / 60 );
    stop(6) = stop(6) + runTime - 60;
else
    stop(6) = stop(6) + runTime;
end

Acc_Vals = zeros(numIMUs, 3);
Gyr_Vals = zeros(numIMUs, 3);


%% RUN test
i = 1;
while (etime(stop,time)) >= seconds(0)
    %fprintf('Sample #%i\ttime: %2.0f:%2.0f \t stop @ %2.0f:%2.0f\n',i,time(5),time(6),stop(5),stop(6))
    if ( rem(i,30) == 0 )
        fprintf('Sample #%i\ttime: %2.0f:%2.0f \t stop @ %2.0f:%2.0f\n',i,time(5),time(6),stop(5),stop(6))
    end
    
    
    %Read raw data & split
    if numIMUs >= 1
        [sensorReadings1,overrun1] = read(imu1);
        Acc_Vals(1,1:3) = sensorReadings1.Acceleration;
        Gyr_Vals(1,1:3) = sensorReadings1.AngularVelocity;
    end
    if numIMUs >= 2
        [sensorReadings2,overrun2] = read(imu2);
        Acc_Vals(2,1:3) = sensorReadings2.Acceleration;
        Gyr_Vals(2,1:3) = sensorReadings2.AngularVelocity;
    end
    if numIMUs >= 3
        [sensorReadings3,overrun3] = read(imu3);
        Acc_Vals(3,1:3) = sensorReadings3.Acceleration;
        Gyr_Vals(3,1:3) = sensorReadings3.AngularVelocity;
    end
    if numIMUs >= 4
        [sensorReadings4,overrun4] = read(imu4);
        Acc_Vals(4,1:3) = sensorReadings4.Acceleration;
        Gyr_Vals(4,1:3) = sensorReadings4.AngularVelocity;
    end
    if numIMUs >= 5
        [sensorReadings5,overrun5] = read(imu5);
        Acc_Vals(5,1:3) = sensorReadings5.Acceleration;
        Gyr_Vals(5,1:3) = sensorReadings5.AngularVelocity;
    end
    
    
    for acc = 1:numIMUs
        for a = 1:3
            if Acc_Vals(acc,a) > 32768
                Acc_Vals(acc,a) = Acc_Vals(acc,a) - 65536;
            end
        end
    end
    for gyr = 1:numIMUs
        for g = 1:3
            if Gyr_Vals(gyr,g) > 32768
                Gyr_Vals(gyr,g) = Gyr_Vals(gyr,g) - 65536;
            end
        end
    end

    for fix = 1:numIMUs
        % Fix
        Ax(fix,i) = double(Acc_Vals(fix,1));
        Ay(fix,i) = double(Acc_Vals(fix,2));
        Az(fix,i) = double(Acc_Vals(fix,3));
        % / 16384.0

        Gx(fix,i) = double(Gyr_Vals(fix,1)*(180/pi));
        Gy(fix,i) = double(Gyr_Vals(fix,2)*(180/pi));
        Gz(fix,i) = double(Gyr_Vals(fix,3)*(180/pi)); 
        % / 131.0);
    end

    % Print
    %fprintf('Gx=%.2f °/s\t Gy=%.2f °/s\t Gy=%.2f °/s\n',Gx(i),Gy(i),Gz(i))
    %fprintf('Ax=%.4f g\t Ay=%.4f g\t Az=%.4f g\n\n',Ax(i),Ay(i),Az(i))

    t(i) = etime(time, strt);
    
    if numIMUs >= 1
        figure(1);
        hold on
        addpoints(axl1, t(i), Ax(1,i));
        addpoints(ayl1, t(i), Ay(1,i));
        addpoints(azl1, t(i), Az(1,i));

        addpoints(gxl1, t(i), Gx(1,i))
        addpoints(gyl1, t(i), Gy(1,i))
        addpoints(gzl1, t(i), Gz(1,i))
        hold off
    end
    if numIMUs >= 2
        figure(2);
        hold on
        addpoints(axl2, t(i), Ax(2,i));
        addpoints(ayl2, t(i), Ay(2,i));
        addpoints(azl2, t(i), Az(2,i));

        addpoints(gxl2, t(i), Gx(2,i))
        addpoints(gyl2, t(i), Gy(2,i))
        addpoints(gzl2, t(i), Gz(2,i))
        hold off
    end
    if numIMUs >= 3
        figure(3);
        hold on
        addpoints(axl3, t(i), Ax(3,i));
        addpoints(ayl3, t(i), Ay(3,i));
        addpoints(azl3, t(i), Az(3,i));

        addpoints(gxl3, t(i), Gx(3,i))
        addpoints(gyl3, t(i), Gy(3,i))
        addpoints(gzl3, t(i), Gz(3,i))
        hold off
    end
    if numIMUs >= 4
        figure(4);
        hold on
        addpoints(axl4, t(i), Ax(4,i));
        addpoints(ayl4, t(i), Ay(4,i));
        addpoints(azl4, t(i), Az(4,i));

        addpoints(gxl4, t(i), Gx(4,i))
        addpoints(gyl4, t(i), Gy(4,i))
        addpoints(gzl4, t(i), Gz(4,i))
        hold off
    end
    if numIMUs >= 5
        figure(5);
        hold on
        addpoints(axl5, t(i), Ax(5,i));
        addpoints(ayl5, t(i), Ay(5,i));
        addpoints(azl5, t(i), Az(5,i));

        addpoints(gxl5, t(i), Gx(5,i))
        addpoints(gyl5, t(i), Gy(5,i))
        addpoints(gzl5, t(i), Gz(5,i))
        hold off
    end
    
    pause(sample)
    time = clock;
    i = i+1;
end


%% SAVE to file
fprintf('\n~Attempting to save the accumulated data~\n')
fileName = "testFiles/test" + int2str(testNr) + ".mat";
folderStatus = mkdir('/testFiles');

EV = ones(1,length(t));

% Output file structure:
% if 4 IMUs are used; x = 4 etc...
%   -> timestamps (1 line)
%   -> Accelerometer X-axis (all IMUs) (x lines)
%   -> Accelerometer Y-axis (all IMUs) (x lines)
%   -> Accelerometer Z-axis (all IMUs) (x lines)
%   -> Gyroscope X-axis (all IMUs) (x lines)
%   -> Gyroscope Y-axis (all IMUs) (x lines)
%   -> Gyroscope Z-axis (all IMUs) (x lines)
outPut = [t; Ax; Ay; Az; Gx; Gy; Gz];

save(fileName, 'outPut');
fprintf('->Data Saved Successfully @ %s\n',fileName)


%% EOF